@@ -2,23 +2,26 @@
 Keyring implementation support
 """
 
-import os
+from __future__ import annotations
+
 import abc
+import copy
 import logging
 import operator
+import os
+import typing
 
-from typing import Optional
-
-import importlib_metadata as metadata
+from jaraco.functools import once
 
 from . import credentials, errors, util
-from .util import properties
+from .compat import properties
+from .compat.py312 import metadata
 
 log = logging.getLogger(__name__)
 
 
 by_priority = operator.attrgetter('priority')
-_limit = None
+_limit: typing.Callable[[KeyringBackend], bool] | None = None
 
 
 class KeyringBackendMeta(abc.ABCMeta):
@@ -44,8 +47,8 @@ class KeyringBackend(metaclass=KeyringBackendMeta):
     def __init__(self):
         self.set_properties_from_env()
 
-    # @abc.abstractproperty
-    def priority(cls):
+    @properties.classproperty
+    def priority(self) -> float:
         """
         Each backend class must supply a priority, a number (float or integer)
         indicating the priority of the backend relative to all other backends.
@@ -60,24 +63,25 @@ class KeyringBackend(metaclass=KeyringBackendMeta):
         As a rule of thumb, a priority between zero but less than one is
         suitable, but a priority of one or greater is recommended.
         """
+        raise NotImplementedError
 
-    @properties.ClassProperty
-    @classmethod
+    @properties.classproperty
     def viable(cls):
         with errors.ExceptionRaisedContext() as exc:
-            cls.priority
+            cls.priority  # noqa: B018
         return not exc
 
     @classmethod
-    def get_viable_backends(cls):
+    def get_viable_backends(
+        cls: type[KeyringBackend],
+    ) -> filter[type[KeyringBackend]]:
         """
         Return all subclasses deemed viable.
         """
         return filter(operator.attrgetter('viable'), cls._classes)
 
-    @properties.ClassProperty
-    @classmethod
-    def name(cls):
+    @properties.classproperty
+    def name(cls) -> str:
         """
         The keyring name, suitable for display.
 
@@ -85,16 +89,14 @@ class KeyringBackend(metaclass=KeyringBackendMeta):
         """
         parent, sep, mod_name = cls.__module__.rpartition('.')
         mod_name = mod_name.replace('_', ' ')
-        return ' '.join([mod_name, cls.__name__])
+        return ' '.join([mod_name, cls.__name__])  # type: ignore
 
-    def __str__(self):
+    def __str__(self) -> str:
         keyring_class = type(self)
-        return "{}.{} (priority: {:g})".format(
-            keyring_class.__module__, keyring_class.__name__, keyring_class.priority
-        )
+        return f"{keyring_class.__module__}.{keyring_class.__name__} (priority: {keyring_class.priority:g})"
 
     @abc.abstractmethod
-    def get_password(self, service: str, username: str) -> Optional[str]:
+    def get_password(self, service: str, username: str) -> str | None:
         """Get password of the username for the service"""
         return None
 
@@ -124,8 +126,8 @@ class KeyringBackend(metaclass=KeyringBackendMeta):
     def get_credential(
         self,
         service: str,
-        username: Optional[str],
-    ) -> Optional[credentials.SimpleCredential]:
+        username: str | None,
+    ) -> credentials.Credential | None:
         """Gets the username and password for the service.
         Returns a Credential instance.
 
@@ -140,18 +142,23 @@ class KeyringBackend(metaclass=KeyringBackendMeta):
                 return credentials.SimpleCredential(username, password)
         return None
 
-    def set_properties_from_env(self):
+    def set_properties_from_env(self) -> None:
         """For all KEYRING_PROPERTY_* env var, set that property."""
 
-        def parse(item):
+        def parse(item: tuple[str, str]):
             key, value = item
             pre, sep, name = key.partition('KEYRING_PROPERTY_')
             return sep and (name.lower(), value)
 
-        props = filter(None, map(parse, os.environ.items()))
+        props: filter[tuple[str, str]] = filter(None, map(parse, os.environ.items()))
         for name, value in props:
             setattr(self, name, value)
 
+    def with_properties(self, **kwargs: typing.Any) -> KeyringBackend:
+        alt = copy.copy(self)
+        vars(alt).update(kwargs)
+        return alt
+
 
 class Crypter:
     """Base class providing encryption and decryption"""
@@ -177,18 +184,16 @@ class NullCrypter(Crypter):
         return value
 
 
-def _load_plugins():
+def _load_plugins() -> None:
     """
     Locate all setuptools entry points by the name 'keyring backends'
     and initialize them.
     Any third-party library may register an entry point by adding the
-    following to their setup.py::
+    following to their setup.cfg::
 
-        entry_points = {
-            'keyring.backends': [
-                'plugin_name = mylib.mymodule:initialize_func',
-            ],
-        },
+        [options.entry_points]
+        keyring.backends =
+            plugin_name = mylib.mymodule:initialize_func
 
     `plugin_name` can be anything, and is only used to display the name
     of the plugin at initialization time.
@@ -205,8 +210,8 @@ def _load_plugins():
             log.exception(f"Error initializing plugin {ep}.")
 
 
-@util.once
-def get_all_keyring():
+@once
+def get_all_keyring() -> list[KeyringBackend]:
     """
     Return a list of all implemented keyrings that can be constructed without
     parameters.
@@ -215,3 +220,43 @@ def get_all_keyring():
     viable_classes = KeyringBackend.get_viable_backends()
     rings = util.suppress_exceptions(viable_classes, exceptions=TypeError)
     return list(rings)
+
+
+class SchemeSelectable:
+    """
+    Allow a backend to select different "schemes" for the
+    username and service.
+
+    >>> backend = SchemeSelectable()
+    >>> backend._query('contoso', 'alice')
+    {'username': 'alice', 'service': 'contoso'}
+    >>> backend._query('contoso')
+    {'service': 'contoso'}
+    >>> backend.scheme = 'KeePassXC'
+    >>> backend._query('contoso', 'alice')
+    {'UserName': 'alice', 'Title': 'contoso'}
+    >>> backend._query('contoso', 'alice', foo='bar')
+    {'UserName': 'alice', 'Title': 'contoso', 'foo': 'bar'}
+    """
+
+    scheme = 'default'
+    schemes = dict(
+        default=dict(username='username', service='service'),
+        KeePassXC=dict(username='UserName', service='Title'),
+    )
+
+    def _query(
+        self, service: str, username: str | None = None, **base: typing.Any
+    ) -> dict[str, str]:
+        scheme = self.schemes[self.scheme]
+        return dict(
+            {
+                scheme['username']: username,
+                scheme['service']: service,
+            }
+            if username is not None
+            else {
+                scheme['service']: service,
+            },
+            **base,
+        )
