@@ -1,26 +1,24 @@
 import logging
 
-from .._compat import properties
 from ..backend import KeyringBackend
+from ..compat import properties
 from ..credentials import SimpleCredential
-from ..errors import PasswordDeleteError, ExceptionRaisedContext
-
+from ..errors import ExceptionRaisedContext, PasswordDeleteError
 
 with ExceptionRaisedContext() as missing_deps:
     try:
         # prefer pywin32-ctypes
-        from win32ctypes.pywin32 import pywintypes
-        from win32ctypes.pywin32 import win32cred
+        from win32ctypes.pywin32 import pywintypes, win32cred
 
         # force demand import to raise ImportError
-        win32cred.__name__
+        win32cred.__name__  # noqa: B018
     except ImportError:
         # fallback to pywin32
         import pywintypes
         import win32cred
 
         # force demand import to raise ImportError
-        win32cred.__name__
+        win32cred.__name__  # noqa: B018
 
 log = logging.getLogger(__name__)
 
@@ -39,7 +37,7 @@ class Persistence:
         if isinstance(value, str):
             attr = 'CRED_PERSIST_' + value.replace(' ', '_').upper()
             value = getattr(win32cred, attr)
-        setattr(keyring, '_persist', value)
+        keyring._persist = value
 
 
 class DecodingCredential(dict):
@@ -81,7 +79,7 @@ class WinVaultKeyring(KeyringBackend):
     persist = Persistence()
 
     @properties.classproperty
-    def priority(cls) -> int:
+    def priority(cls) -> float:
         """
         If available, the preferred backend on Windows.
         """
