@@ -1,13 +1,19 @@
-#!/usr/bin/env python
 """Simple command line interface to get/set password from a keyring"""
 
-import getpass
 import argparse
+import getpass
 import sys
 
-from . import core
-from . import backend
-from . import set_keyring, get_password, set_password, delete_password
+from . import (
+    backend,
+    completion,
+    core,
+    delete_password,
+    get_password,
+    set_keyring,
+    set_password,
+)
+from .util import platform_
 
 
 class CommandLineTool:
@@ -35,9 +41,10 @@ class CommandLineTool:
         self.parser.add_argument(
             "--disable", action="store_true", help="Disable keyring and exit"
         )
+        self.parser._operations = ["get", "set", "del", "diagnose"]
         self.parser.add_argument(
             'operation',
-            help="get|set|del",
+            choices=self.parser._operations,
             nargs="?",
         )
         self.parser.add_argument(
@@ -48,6 +55,7 @@ class CommandLineTool:
             'username',
             nargs="?",
         )
+        completion.install(self.parser)
 
     def run(self, argv):
         args = self.parser.parse_args(argv)
@@ -62,6 +70,10 @@ class CommandLineTool:
             core.disable()
             return
 
+        if args.operation == 'diagnose':
+            self.diagnose()
+            return
+
         self._check_args()
         self._load_spec_backend()
         method = getattr(self, f'do_{self.operation}', self.invalid_op)
@@ -85,13 +97,18 @@ class CommandLineTool:
         set_password(self.service, self.username, password)
 
     def do_del(self):
-        self.input_password(
-            f"Deleting password for '{self.username}' in '{self.service}': "
-        )
         delete_password(self.service, self.username)
 
+    def diagnose(self):
+        config_root = core._config_path()
+        if config_root.exists():
+            print("config path:", config_root)
+        else:
+            print("config path:", config_root, "(absent)")
+        print("data root:", platform_.data_root())
+
     def invalid_op(self):
-        self.parser.error("Specify operation 'get', 'del', or 'set'.")
+        self.parser.error(f"Specify operation ({', '.join(self.parser._operations)}).")
 
     def _load_spec_backend(self):
         if self.keyring_backend is None:
@@ -101,7 +118,7 @@ class CommandLineTool:
             if self.keyring_path:
                 sys.path.insert(0, self.keyring_path)
             set_keyring(core.load_keyring(self.keyring_backend))
-        except (Exception,) as exc:
+        except Exception as exc:
             # Tons of things can go wrong here:
             #   ImportError when using "fjkljfljkl"
             #   AttributeError when using "os.path.bar"
@@ -121,8 +138,15 @@ class CommandLineTool:
 
     @staticmethod
     def strip_last_newline(str):
-        """Strip one last newline, if present."""
-        return str[: -str.endswith('\n')]
+        r"""Strip one last newline, if present.
+
+        >>> CommandLineTool.strip_last_newline('foo')
+        'foo'
+        >>> CommandLineTool.strip_last_newline('foo\n')
+        'foo'
+        """
+        slc = slice(-1 if str.endswith('\n') else None)
+        return str[slc]
 
 
 def main(argv=None):
