@@ -3,14 +3,14 @@ Core API functions and initialization routines.
 """
 
 import configparser
+import logging
 import os
 import sys
-import logging
 import typing
 
 from . import backend, credentials
-from .util import platform_ as platform
 from .backends import fail
+from .util import platform_ as platform
 
 LimitCallable = typing.Callable[[backend.KeyringBackend], bool]
 
@@ -134,7 +134,7 @@ def load_keyring(keyring_name: str) -> backend.KeyringBackend:
     """
     class_ = _load_keyring_class(keyring_name)
     # invoke the priority to ensure it is viable, or raise a RuntimeError
-    class_.priority
+    class_.priority  # noqa: B018
     return class_()
 
 
